# -*- coding: utf-8 -*-
"""
Created on Thu Jan 10 08:43:11 2019

@author: Machacek
"""

import numpy as np
import matplotlib.pyplot as plt
from matplotlib import cm
from matplotlib import gridspec
import os
import itertools

plt.rcParams['xtick.direction'] = 'in'
plt.rcParams['ytick.direction'] = 'in'

# -----------------------------------------------------------------------------
# Color definitions (RGB)
# -----------------------------------------------------------------------------
                        
# Color schemes generated with "internal" color maps          
start = 0.0
stop = 1.0
number_of_lines= 6

coolwarm = [ cm.coolwarm(x) for x in np.linspace(start, stop, number_of_lines)  ]


# -----------------------------------------------------------------------------
# Functions / Definitions
# -----------------------------------------------------------------------------
def cm2inch(*tupl):
    inch = 2.54
    if isinstance(tupl[0], tuple):
        return tuple(i/inch for i in tupl[0])
    else:
        return tuple(i/inch for i in tupl)

# -----------------------------------------------------------------------------
# Enter folder names
# -----------------------------------------------------------------------------
    
folder_list_2D = ['u3p3-sat-mini', 'u4p4-sat-mini','u4p4-sat-mini-red','u4p4u4','u8p4-sat','u8p4-sat-red', 'u8p4u8','u4u4','u8u8'] #,
folder_list_3D = ['u10p4-sat', 'u20p8-sat', 'u4p4-sat-3d-mini', 'u20p8-sat-red', 'u20p8u20', 'u20u20', 'u27p8-sat']

nfolders = len(folder_list_2D) + len(folder_list_3D)

# -----------------------------------------------------------------------------
# Analytical data
# -----------------------------------------------------------------------------
fname = 'analytical/Terzaghi_p_t01.dat'
tmp = np.loadtxt(fname, skiprows=0 )
pw_a_t0  = tmp[:]  
fname = 'analytical/Terzaghi_p_t1.dat'
tmp = np.loadtxt(fname, skiprows=0 )
depth_a  = tmp[:,1] 
pw_a_t1  = tmp[:,0]    
fname = 'analytical/Terzaghi_p_t10.dat'
tmp = np.loadtxt(fname, skiprows=0 )
pw_a_t2  = tmp 
fname = 'analytical/Terzaghi_p_t100.dat'
tmp = np.loadtxt(fname, skiprows=0 )
pw_a_t3  = tmp
fname = 'analytical/Terzaghi_p_t1000.dat'
tmp = np.loadtxt(fname, skiprows=0 )
pw_a_t4  = tmp
fname = 'analytical/Terzaghi_p_t10000.dat'
tmp = np.loadtxt(fname, skiprows=0 )
pw_a_t5  = tmp
depth_a = depth_a[::-1]

fname = 'analytical/Terzaghi_U.dat'
tmp = np.loadtxt(fname, skiprows=0 )
time_a = tmp[:,0] 
u2_a   = tmp[:,1] * 100. 


# -----------------------------------------------------------------------------
# Plotting ranges
# -----------------------------------------------------------------------------

xmax = 60.
xmin = 0.0
ymax = 1.

# Begin figure
fig = plt.figure(figsize=cm2inch(18,9*nfolders)) 

# Set the layout of the subplots
gs = gridspec.GridSpec(nfolders,2)

time_steps = [0.05,0.1,1,10,100,1000,10000]
label_time_steps = ['$t = 5 \cdot10^{-2}$ s','$t = 10^{-1}$ s','$t = 10^{0}$ s',\
                    '$t = 10^{1}$ s','$t = 10^2$ s','$t = 10^3$ s','$t = 10^4$ s']


i = 0
count = 0
file_count = 0
for name in itertools.chain(folder_list_2D, folder_list_3D):

    vertical_coords = []
    pore_pressure  = []
    displacement = []
    time = []
    data = []

# =============================================================================
# Get all data files for each calculation and store them in vertical_coords and pore_pressure
# =============================================================================
    for filename in os.listdir(os.getcwd()+'/'+name+ '/'+'terzaghi-print-out'):

        data = np.genfromtxt(os.getcwd()+'/'+name+ '/'+'terzaghi-print-out/' + filename, skip_header=1, skip_footer=0)
        time = np.genfromtxt(os.getcwd()+'/'+name+ '/'+'terzaghi-print-out/' + filename, skip_header=1, skip_footer=0, usecols=0) -1.
        
        if (filename[0:4] == 'LEFT' or filename[0:3] == 'ALL' or filename[0:4] == 'SIDE' ):
            
            for itime_step in time_steps:

               row =min(range(len(time)), key=lambda j: abs(time[j]-itime_step))
               
               if(file_count>len(folder_list_2D)-1):
                   vertical_coords.append(data[row,3]) # coords
                   pore_pressure.append(data[row,4])   #pore water pressure
               else:
                   vertical_coords.append(data[row,2]) # coords
                   pore_pressure.append(data[row,4])   #pore water pressure

            

        elif (filename[0:3] == 'TOP'):
            
            if(file_count>len(folder_list_2D)-1):
                displacement = np.genfromtxt(os.getcwd()+'/'+name+ '/'+'terzaghi-print-out/' + filename, skip_header=1, skip_footer=0, usecols=3)
            else:
                displacement = np.genfromtxt(os.getcwd()+'/'+name+ '/'+'terzaghi-print-out/' + filename, skip_header=1, skip_footer=0, usecols=2)

            displacement_time = np.genfromtxt(os.getcwd()+'/'+name+ '/'+'terzaghi-print-out/' + filename, skip_header=1, skip_footer=0, usecols=0)
            displacement = (displacement - displacement[0])*100.
         
    count = 0

# =============================================================================
# Now plot data for each time step for all nodes/elements along the height
# =============================================================================
    local_vertical_coords = []
    local_vertical_coords2 =[]
    local_vertical_coords = vertical_coords[0::len(time_steps)]

    indeces = [u[0] for u in sorted(enumerate(local_vertical_coords), key=lambda x:x[1])]
    local_vertical_coords2 = sorted(local_vertical_coords,reverse=False)

    for itime_step in time_steps:
        
        local_pore_pressure = []
        local_pore_pressure2 = []
        local_pore_pressure = pore_pressure[count::len(time_steps)]
        
        for x in range(len(local_pore_pressure)):
            scalar = local_pore_pressure[indeces[x]]
            local_pore_pressure2.append(scalar)
        
        for j in range(len(local_pore_pressure2)):
            local_pore_pressure2[j] -= (1.-local_vertical_coords2[j])*9.81
        
        plt.subplot(gs[i])
        plt.title(name,fontsize=11)
        
        if(count == 0) :
            plt.plot(pw_a_t0[::2],depth_a[::2], linestyle='None', marker = 'o', markeredgecolor=coolwarm[0],markerfacecolor='white', lw=1.0)
        if(count == 1) :
            plt.plot(pw_a_t1[::2],depth_a[::2], linestyle='None', marker = 'o', markeredgecolor=coolwarm[1],markerfacecolor='white', lw=1.0)
        if(count == 2) :
            plt.plot(pw_a_t2[::2],depth_a[::2], linestyle='None', marker = 'o', markeredgecolor=coolwarm[2],markerfacecolor='white', lw=1.0)
        if(count == 3) :
            plt.plot(pw_a_t3[::2],depth_a[::2], linestyle='None', marker = 'o', markeredgecolor=coolwarm[3],markerfacecolor='white', lw=1.0)
        if(count == 4) :
            plt.plot(pw_a_t4[::2],depth_a[::2], linestyle='None', marker = 'o', markeredgecolor=coolwarm[4],markerfacecolor='white', lw=1.0)
        if(count == 5) :
            plt.plot(pw_a_t5[::2],depth_a[::2], linestyle='None', marker = 'o', markeredgecolor=coolwarm[5],markerfacecolor='white', lw=1.0)
            
        if(count == 0):
            plt.plot(local_pore_pressure2,local_vertical_coords2,color='k', lw=1.0, label=label_time_steps[count])
        else:
            plt.plot(local_pore_pressure2,local_vertical_coords2,color=coolwarm[count-1], lw=1.0, label=label_time_steps[count])
            
            
        plt.xlim(0,xmax)
        plt.ylim(0,ymax)
        plt.xlabel('Excess pore-water pressure $\Delta p_w$ [kPa]' )
        plt.ylabel('Depth t [m]')
        plt.legend()
        plt.grid(True)
        
        count += 1
                
# =============================================================================
# Plot displacement
# =============================================================================
    plt.subplot(gs[i+1])
    plt.title(name,fontsize=11)
    plt.semilogx(time_a[::6],u2_a[::6], linestyle='None', marker = 'o', markeredgecolor=coolwarm[0],markerfacecolor='white', lw=1.0, label='Analytical')
    plt.semilogx(displacement_time-1, displacement, color=coolwarm[0], lw=1.0, label='Simulation')
    plt.xlim(0.01,10000)
    plt.ylim(-7,0.)
    plt.xlabel('Time $t$ [s]' )
    plt.ylabel('Vertical displacement $u_2$ [cm]')
    plt.legend(loc='lower left')
    plt.grid(True)
        
    i = i + 2
    file_count += 1


plt.tight_layout(w_pad=1.5)
plt.show()

fig.savefig('terzaghi_comparison.pdf', bbox_inches='tight')
#fig.savefig('terzaghi_comparison.svg', bbox_inches='tight')